/*
 *  ProjectileModel.cpp
 *
 *  Created by Roy Fejgin on 2/3/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "ProjectileModel.h"
#import "my_util.h"

void ProjectileModel::addProjectile(CGPoint sling1, CGPoint sling2, CGPoint proj)
{
    DEBUG_LOG(@"addProjectile:  p1.x %f, p1.y %f, p2.x %f, p2.y %f p3.x %f p3.y %f", 
          proj.x,proj.y,sling1.x,sling1.y,sling2.x,sling2.y);

    
    Vector3D projVec = Vector3D(proj.x, proj.y, 0);
    Vector3D v1 = Vector3D(sling1.x, sling1.y, 0);
    Vector3D v2 = Vector3D(sling2.x, sling2.y, 0);
    
    Vector3D temp = v1-v2;
    DEBUG_LOG(@"temp: %f, %f", temp.x,temp.y);
    temp *= 0.5;
    Vector3D targetVec = v2+temp;
    Vector3D direction = targetVec - projVec;
        
    Projectile* final = new Projectile();
    
    //direction.normalize();
    final->direction = direction*.1;
    final->location = projVec;
    final->radius = m_xMax/8.;
    final->TTL = DefaultTTL;
    
    // save in DB
    m_proj.push_back(final);
    
}

bool ProjectileModel::simulateCollision(Projectile* proj1, Projectile* proj2, Collision* collision)
{
    bool didCollide = false;

    Vector3D l1 = proj1->location + proj1->direction;
    Vector3D l2 = proj2->location+ proj2->direction;
    Vector3D v1 = proj1->direction;
    Vector3D v2 = proj2->direction;
    
    // Ignore if they are already overlapping
    if(computeDistance3D(proj1->location, proj2->location) <
       (proj1->radius+proj2->radius)) {
        return false;
    }
    
    // Did they collide?
    if(computeDistance3D(l1, l2) <
       (proj1->radius+proj2->radius))
    {
        DEBUG_LOG(@"Collision!");
        didCollide = true;
        
        // vector connecting center of both projectiles
        Vector3D center = l2 - l1;
 
        // projection of v1 onto line connecting the centers of the two balls
        Vector3D vc1 = project(v1,center);
        Vector3D vc2 = project(v2,center);
        
        // normal to the previous projection
        Vector3D norm1 = project(v1, orthoVec(vc1));
        Vector3D norm2 = project(v2, orthoVec(vc2));        
        
        // new force along the line connecting the centers
        Vector3D newVc1 = (vc1+vc2);
        newVc1 *= 0.5;
        Vector3D newVc2 = (vc1+vc2);
        newVc2 *= -0.5;
        
        // Save new direction
        proj1->direction = newVc1+norm1;
        proj2->direction = newVc2+norm2;
        
        // Decrement time-to-live
        if(proj1->TTL > 0 ) {
            proj1->TTL--;
        }
        if(proj2->TTL > 0 ) {
            proj2->TTL--;
        }
        
        // Return collision location
        Vector3D offset = vc1;
        offset.normalize();
        offset *= proj1->radius;
        Vector3D result = l1+offset;
        collision->location.x = result.x;
        collision->location.y = result.y;
        if((vc1.magnitude() + vc2.magnitude()) > ExplosionThresh) {
            collision->type = COL_EXPLOSION;
        } 
        else {
            collision->type =COL_BOUNCE;
        }

     }
    
    return didCollide;
}


ColMetaData ProjectileModel::advance()
{
    ColMetaData colData = {false, false};
    
    // clear old list of collisions
    m_collisions.clear();
    for( list<Projectile*>::iterator iter = m_proj.begin(); iter!= m_proj.end(); ++iter) {
        Projectile* proj  = (*iter);
        Vector3D direction = proj->direction;
        Collision col;
        
        // Detect collision with X axis wall
        if(((proj->location.x + direction.x + proj->radius >= m_xMax) && (direction.x > 0)) || 
           ((proj->location.x + direction.x - proj->radius <= -m_xMax) && (direction.x < 0))) {
            if (proj->TTL > 0) {
                proj->TTL--;
                // if the projctile exceeded its TTL, let it drift out of bounds
                proj->direction.x *= -1;
                colData.bounceCollision = true;
            }
        }
        
        // Detect collision with Y axis wall
        if(((proj->location.y + direction.y + proj->radius >= m_yMax) && (direction.y > 0)) || 
           ((proj->location.y +direction.y - proj->radius <= -m_yMax) && (direction.y < 0))) {
            if (proj->TTL > 0) {
                proj->TTL--;
                // if the projctile exceeded its TTL, let it drift out of bounds
                proj->direction.y *= -1;
                colData.bounceCollision = true;
            }
        }
        
        if(colData.bounceCollision) {
            // rough location
            col.location.x = proj->location.x;
            col.location.y = proj->location.y;
            col.type = COL_WALL;
            m_collisions.push_back(col);
        }
        
        bool shouldDelete = false;
        // Check for collision with all other projectiles
        for( list<Projectile*>::iterator iter2 = m_proj.begin(); iter2!= m_proj.end(); ++iter2) {
            Projectile* proj2 = (*iter2);
            Vector3D direction2 = proj2->direction;
            // ignore collision with self!
            if (proj2 == proj) {
                continue; 
            }
            
            // Collision detection and simulation
            if(simulateCollision(proj, proj2, &col)) {
                m_collisions.push_back(col);
                if (col.type == COL_EXPLOSION) {
                    colData.expCollision = true;
                    
                    // Explosion ==> Remove both projectiles from the simulation
                    shouldDelete = true;
                    //delete proj;
                    //proj = NULL;
                    //iter = m_proj.erase(iter);
                    delete proj2;
                    proj2 = NULL;
                    iter2 = m_proj.erase(iter2);
                }
                else if (col.type == COL_BOUNCE ) {
                    colData.bounceCollision = true;
                }
            }
        }
        
        if (shouldDelete) {
            delete proj;
            proj = NULL;
            iter = m_proj.erase(iter);
        }
        
        if (proj != NULL) {
            // reload direction since it may have changed
            direction = proj->direction;
            
            // Move projectile
            proj->location += direction;

            
            // remove projectiles that are out of the screen
            if (!isPointInScreen(proj->location.x, proj->location.y, proj->radius)) {
                DEBUG_LOG(@"Removing projectile out of screen bounds, addr= 0x%x", proj);
            
                delete proj;
                iter = m_proj.erase(iter); 
            }
        }
    }
           
    return colData;
}

list<ExportedProjectile> ProjectileModel::getProjectiles()
{
    list<ExportedProjectile> points;
    
    for( list<Projectile*>::iterator iter = m_proj.begin(); iter!= m_proj.end(); ++iter) {
        ExportedProjectile proj;
        CGPoint p;
        p.x = (*iter)->location.x;
        p.y = (*iter)->location.y;
        proj.location = p;
        proj.radius = (*iter)->radius;
        points.push_back(proj);
    }
    
    return points;
}

list<Collision> ProjectileModel::getCollisions()
{
    return m_collisions;
}

