/*
 *  SlingModel.h
 *  Model of a slingshot. 
 *  This class tracks incoming touches, and uses those to transition the slingshot
 *  between the following states: off, idle, streched, vibrating, 
 *  and sling (ready to shoot).
 *
 *  Created by Roy Fejgin on 1/31/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *  
 */


#import "my_defs.h"
#import "mo_touch.h"


#ifndef __SLING_MODEL__
#define __SLING_MODEL__


typedef enum SlingState {
    SL_OFF = 0,
    SL_IDLE,
    SL_STRECHED,
    SL_VIBRATING,
    SL_SLING,
    SL_LAST
}SlingState;


typedef struct SlingEvent {
    bool pluck;
    bool shot;
    CGPoint shotTouches[3];
} SlingEvent;

typedef struct TouchData
{
    long handle;
    GLfloat x;
    GLfloat y;
} TouchData;


class SlingModel {
public:
    SlingModel(): m_freq(440), m_numTouches(0), m_state(SL_OFF) {
        // initialize touch data
        for(int i=0; i<3; i++) {
            m_touchData[i].x = m_touchData[i].y = 0;
            m_touchData[i].handle = 0;
        }
    }
    ~SlingModel(){}
    SlingState getSlingState() { return m_state; }
    CGPoint getPoint(int n) {
        CGPoint point;
        assert(n < m_numTouches);
        
        point.x = m_touchData[n].x;
        point.y = m_touchData[n].y;
        return point;
    }
    SlingEvent touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * views);
    
    GLfloat getFreq() { return m_freq; }

private:
    bool hasTouch(UITouch* pTouch) {
        for (int i=0; i<m_numTouches; i++) {
            if ((m_touchData[i].handle) == (long)pTouch) {
                return true;
            }
        }
        return false;
    }
    
    void saveTouch(UITouch* pTouch, CGPoint* pPoint, int touchNum);
    void clearTouch(int touchNum);
    GLfloat computeFreq();
    void dumpTouchData();
    bool isTouchOnCord();
    bool copyTouches(SlingEvent* pEvent);

    
private:
    
    SlingState m_state;
    
    TouchData m_touchData[3];
    
    int m_numTouches;
    int m_freq;
};





#endif //__SLING_MODEL__
