/*
 *  SlingModel.cpp
 *  Sonic
 *
 *  Created by Roy Fejgin on 1/31/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "SlingModel.h"
#import "my_util.h"

using namespace std;

/*
typedef enum SlingState {
    SL_OFF = 0,
    SL_IDLE,
    SL_STRECHED,
    SL_VIBRATING,
    SL_RELEASING,
    SL_LAST
}SlingState;
*/
/*
SlingState touchRemoveTransitions[3][3] =
{
    {SL_OFF, SL_LAST, SL_LAST},
    {SL_IDLE, SL_IDLE, SL_LAST},
    {SL_STRECHED, SL_STRECHED, SL_STRECHED}
};
*/
void SlingModel::saveTouch(UITouch* pTouch, CGPoint* pPoint, int touchNum)
{
    assert(touchNum<3);
    m_touchData[touchNum].x = pPoint->x;
    m_touchData[touchNum].y = pPoint->y;
    m_touchData[touchNum].handle = (long)pTouch;
}

void SlingModel::clearTouch(int touchNum)
{
    m_touchData[touchNum].handle = 0;
}

void dumpTouchMap(const std::map<int, MoTouchTrack> & touchPts)
{
    DEBUG_LOG(@"\nDumping callback touch map:");
    
    for(MoTouch::MoTouchMapIt  iter = touchPts.begin(); iter != touchPts.end(); ++iter) {
        UITouch* pTouch = iter->second.touch;
        DEBUG_LOG(@"Addr = 0x%x\t Phase = %d", pTouch, [pTouch phase]);
    }
    DEBUG_LOG(@"\n");
}

void SlingModel::dumpTouchData()
{
#ifdef DEBUG_ON
    DEBUG_LOG(@"\n***START private touch data:***");
    DEBUG_LOG(@"Num touches: %d", m_numTouches);
    for (int i = 0; i<3; i++) {
        DEBUG_LOG(@"Index: %d, address: 0x%x", i, m_touchData[i].handle);
    }
    DEBUG_LOG(@"\n***END private touch data ***");
#endif
}

bool SlingModel::copyTouches(SlingEvent* pEvent)
{
    for (int i = 0; i < 3 ; i++) {
        pEvent->shotTouches[i].x = m_touchData[i].x;
        pEvent->shotTouches[i].y = m_touchData[i].y;
    }
}

SlingEvent SlingModel::touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * view)
{
    UIView * vview = (UIView *)view;
    SlingEvent retVal;
    retVal.pluck = false;
    retVal.shot = false;

    DEBUG_LOG(@"Touch Event enter");
    dumpTouchMap(touchPts);
    
    // Process changes to existing touches
    for(int i=0; i < m_numTouches; i++) {
        // Does our touch exist in the new map?
        if(touchPts.find(m_touchData[i].handle) != touchPts.end()) {
            MoTouch::MoTouchMapIt  iter = touchPts.find(m_touchData[i].handle);
            UITouch* pTouch = iter->second.touch;
            CGPoint point = converToWorld([pTouch locationInView:vview]);

            // check phase            
            switch ([pTouch phase]) {
                case UITouchPhaseEnded:
                case UITouchPhaseCancelled:
                    // TODO: callbacks?
                    switch (m_numTouches) {
                        case 1:
                            assert(m_touchData[0].handle == (long) pTouch);
                            DEBUG_LOG(@"touchEvent: 1 => 0, addr=0x%x",pTouch);
                            m_state = SL_OFF;
                            m_numTouches = 0;
                            clearTouch(0);
                            break;
                        case 2:
                            DEBUG_LOG(@"touchEvent: 2 => 1, addr=0x%x",pTouch);
                            m_state = SL_IDLE;
                            m_numTouches = 1;
                            if(i==0) {
                                DEBUG_LOG(@"touchEvent: Moving second touch back, addr=0x%x",pTouch);
                                assert(m_touchData[0].handle == (long) pTouch);
                                // Move second touch one slot back
                                m_touchData[0] = m_touchData[1];
                                i = -1; // the ugliest hack of my life (so far)
                            }
                            else {
                                assert(m_touchData[1].handle == (long) pTouch);
                            }
                            clearTouch(1);
                            break;
                        case 3:
                            // Third touch removed
                            DEBUG_LOG(@"touchEvent: 3 => 2, i == %d, addr=0x%x",i, pTouch);
                            // it was the first sling finger
                            if(i==0) {
                                m_touchData[0] = m_touchData[1];
                                m_touchData[1] = m_touchData[2];                                  
                            }
                            // it was the scecond sling finger
                            else if (i==1) {
                                // convert projectile finger to a sling finger
                                m_touchData[1] = m_touchData[2];                                  
                            }
                            
                            // TODO this logic is wrong
                            if (m_state == SL_SLING) {
                                // Slingshot release
                                if (i==2) {
                                    // Shoot projectile and pluck string
                                    retVal.pluck = true;
                                    retVal.shot = true;
                                    copyTouches(&retVal);
                                    m_freq = computeFreq();
                                    m_state = SL_VIBRATING;
                                }
                                // Sling cancelled => go back to streched
                                else {
                                    m_state = SL_STRECHED;
                                }
                            }
                            // Touch up for a non-sling pluck
                            else {
                                m_state = SL_VIBRATING;
                            }
                            
                            m_numTouches--;
                            clearTouch(2);                            
                            
                            // todo - change state??
                            
                            // if tapping -> only update db
                            // if releasing sling -> do release
                            //assert(0); // not implemented yet
                            break;

                        default:
                            DEBUG_LOG(@"Warning: touch ended event ignore for touch %d", i);
                            break;
                    }
                    break;
                case UITouchPhaseMoved:
                    DEBUG_LOG(@"touchEvent: MOVED addr=0x%x",pTouch);
                    saveTouch(pTouch, &point, i);
                    break;


                default:
                    break;
            }
            dumpTouchData();
        }
    }
    
    // Look for new touches
    for(MoTouch::MoTouchMapIt  iter = touchPts.begin(); iter != touchPts.end(); ++iter) {
        UITouch* pTouch = iter->second.touch;
        CGPoint point = converToWorld([pTouch locationInView:vview]);

        if (([pTouch phase] == UITouchPhaseBegan) && !hasTouch(pTouch)) {
            switch (m_numTouches) {
                case 0:
                    DEBUG_LOG(@"touchEvent: 0 => 1, addr=0x%x",pTouch);
                    m_state = SL_IDLE;
                    m_numTouches = 1;
                    saveTouch(pTouch, &point, 0);
                    break;
                case 1:
                    DEBUG_LOG(@"touchEvent: 1 => 2, addr=0x%x",pTouch);
                    m_state = SL_STRECHED;
                    m_numTouches = 2;
                    saveTouch(pTouch, &point, 1);
                    break;
                case 2:
                    DEBUG_LOG(@"touchEvent: 2 => 3, addr=0x%x",pTouch);
                    saveTouch(pTouch, &point, 2);
                    m_numTouches++;
                    
                    if(isTouchOnCord()) {
                        retVal.pluck = true;
                        m_freq = computeFreq();
                        m_state = SL_VIBRATING;
                    } 
                    else {
                        m_state = SL_SLING;
                    }

                    // todo                    
                    // either tap or sling shot - need to determine which
                    //assert(0); // not implemented yet
                    // 
                    break;
                default:
                    // fourth touch -> replace the third touch
                    saveTouch(pTouch, &point, 2);
                    m_freq = computeFreq();
                    break;
                    
            }
            dumpTouchData();        
        }
    }
    
    DEBUG_LOG(@"Touch Event exit");
    return retVal;
}


GLfloat SlingModel::computeFreq()
{
    assert(m_numTouches>=2);
    CGPoint p1 = {m_touchData[0].x, m_touchData[0].y};
    CGPoint p2 = {m_touchData[1].x, m_touchData[1].y};
    GLfloat dist = computeDistance(p1, p2);
    GLfloat freq = 120 + (dist/2.0)* 800;
    return freq;
}

bool SlingModel::isTouchOnCord()
{
    bool result = true;
    Float32 distIndicator = 0, cordLength = 0;
    CGPoint p1 = {m_touchData[0].x, m_touchData[0].y};
    CGPoint p2 = {m_touchData[1].x, m_touchData[1].y};
    CGPoint p3 = {m_touchData[2].x, m_touchData[2].y};
    
    distIndicator += computeDistance(p1, p3);
    distIndicator += computeDistance(p2, p3);
    cordLength = computeDistance(p1, p2);
    
    if ((distIndicator / cordLength) > 1.07) {
        result = false;
    }

    return result;
}