/*
 *  WorldModel.h
 *  A model of the sonic slingshot instrument.
 *  Includes:
 *     - a model of the actual slingshot
 *     - a model of the projectiles
 *     - audio generation
 *
 *  Does NOT include any of the GUI.
 *    
 *
 *  Created by Roy Fejgin on 1/31/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __WORLD_MODEL__
#define __WORLD_MODEL__

#import "my_defs.h"
#import "mo_touch.h"
#import "SlingModel.h"
#import "PluckedString.h"
#import "mo_gfx.h"
#import "ProjectileModel.h" 
#import "AudioClipPlayer.h"
#import <list>


//typedef void (* StringStartCallback)(GLfloat);



class WorldModel {
public:
    // Constructor
    WorldModel(int sample_rate);
    
    // Destructor
    ~WorldModel(){ 
        delete m_pSling;
        delete m_pKarplus;
        delete m_pProj;
        delete m_pWallAudio;
        delete m_pCollAudio;
    }
    
    void advance(); 
    void touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * view);
    
    SlingState getSlingShotState() { return m_pSling->getSlingState(); } 
    inline CGPoint getSlingPoint(int i) { return m_pSling->getPoint(i); }
    inline GLfloat getStringAmplitude() { 
        if(m_pKarplus != NULL) {
            return stringDecay;
        }
        else {
            return 0;
        }
    };
    
    inline list<ExportedProjectile> getProjectiles() { return m_pProj->getProjectiles(); }
    inline list<Collision> getCollisions() { return m_pProj->getCollisions(); } 
    
    inline SAMPLE tick() {
        GLfloat output = 0.0;
        
        // Play string
        if (m_pKarplus != NULL) {
            m_pKarplus->tick(0);
            output +=  m_pKarplus->getOutput();
            m_pKarplus->endOfTick();
            stringDecay*=.9999;
        }
        
        // Play audio clips
        if (m_pWallAudio != NULL) {
            output += m_pWallAudio->tick();
        }
        if (m_pCollAudio != NULL) {
            output += m_pCollAudio->tick();
        }
        return output;
    }
    
private:
    // slingshot model
    SlingModel* m_pSling;
    
    // plucked string
    PluckedString* m_pKarplus;
    Float32 stringDecay;
    
    // projectile simulation
    ProjectileModel* m_pProj;
    
    // for playing .wav files
    AudioClipPlayer* m_pWallAudio;
    AudioClipPlayer* m_pCollAudio;
    
    // Sample rate
    GLfloat m_sr;
};

#endif // __WORLD_MODEL__