/*
 *  WorldModel.cpp
 *  Sonic
 *
 *  Created by Roy Fejgin on 1/31/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "WorldModel.h"

WorldModel::WorldModel(int sample_rate)
{
    m_pSling = new SlingModel();
    m_pProj = new ProjectileModel(1, 1.5);
    
    const char *collisionPath = [[[NSBundle mainBundle]
                                  pathForResource:@"expl2"
                                  ofType:@"wav"] UTF8String];        
    
    const char *wallPath = [[[NSBundle mainBundle]
                                  pathForResource:@"ball223"
                                  ofType:@"wav"] UTF8String];       
    
    m_pCollAudio = new AudioClipPlayer(collisionPath, 0.25);
    m_pWallAudio = new AudioClipPlayer(wallPath, 0.05);
    m_sr = sample_rate;
}


void WorldModel::advance() {
    
    ColMetaData collisionData;
    // Run simulation and check for collisions
    collisionData = m_pProj->advance();
    
    // Play collision audio cue
    if(collisionData.bounceCollision) {
        m_pWallAudio->playClip();
    }

    // Play audio cue for ball hitting wall (or another ball)
    if(collisionData.expCollision) {
        m_pCollAudio->playClip();
    }
}    

void WorldModel::touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * view)
{    
    SlingEvent slingEvent = m_pSling->touchEvent(touchPts, view);
    
    // New pluck of the string
    if (slingEvent.pluck) {
        stringDecay = 1.0;
        GLfloat freq = m_pSling->getFreq();
        PluckedString* pNewStr = new PluckedString(freq, m_sr, NULL, 0);        
        PluckedString* pOldStr = m_pKarplus;

        // Replace old with new
        m_pKarplus = pNewStr;
        // Todo: locking? the audio thread may be using the object
        delete pOldStr;
    }
    
    if (slingEvent.shot) {
        NSLog(@"slingEvent: shot %d p1.x %f, p1.y %f, p2.x %f, p2.y %f p3.x %f p3.y %f", slingEvent.shot,
              slingEvent.shotTouches[0].x, slingEvent.shotTouches[0].y,slingEvent.shotTouches[1].x, slingEvent.shotTouches[1].y,
              slingEvent.shotTouches[1].x, slingEvent.shotTouches[1].y);
        m_pProj->addProjectile(slingEvent.shotTouches[0], slingEvent.shotTouches[1], slingEvent.shotTouches[2]);
    }
}

/*PluckedString::PluckedString(MyFloat64 freq, unsigned int samplingRate, 
 ReleaseCallback callback, unsigned int callbackUserData)
*/