/*
 *  ui_projectiles.h
 *  
 *  Visual representation of projectiles and collisions.
 *  The projectiles are visualized as balls, and the collisions
 *  as an explosion.
 *
 *  Created by Roy Fejgin on 2/3/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */


#include "ui_element.h"
#include <list>

class UI_Collision: public UI_Element
{
public:    
    UI_Collision(WorldModel* pModel, CGPoint location, GLuint texture) : m_done(false), UI_Element(pModel) {
        m_location = location;
        m_intensity = 1.0;
        m_scale = 1.0;
        m_texture = texture;
    }
    virtual void draw();
    inline bool isDone() { return m_done; }
private:
    const static int TTL = 20;
    
    CGPoint m_location;
    bool m_done;
    GLfloat m_intensity;
    GLfloat m_scale;
    GLuint m_texture;
};

class  UI_Projectiles : public UI_Element
{
public:
    
    UI_Projectiles(WorldModel* pModel, GLuint textureBall, GLuint textureExp) :
        UI_Element(pModel), m_textureBall(textureBall), m_textureExp(textureExp) {}
    ~UI_Projectiles(){}
    
    virtual void draw();
    
private:
    void drawBall(ExportedProjectile ball);
    void drawCollision(Collision col);
    
private:
    list<UI_Collision*> m_uiCollsions;
    GLuint m_textureBall;
    GLuint m_textureExp;
    
};
