/*
 *  ui_projectiles.cpp
 *  
 *
 *  Created by Roy Fejgin on 2/3/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "ui_projectiles.h"



void UI_Projectiles::drawBall(ExportedProjectile ball)
{
    
    GLfloat factor = ball.radius;
    
    GLfloat squareVertices[] = {
        factor*-1.0f, factor*-1.0f,
        factor*1.0f,  factor*-1.0f,
        factor*-1.0f, factor*1.0f,
        factor*1.0f,  factor*1.0f
    };
 /*   
    static const GLubyte squareColors[] = {
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
    };*/
    static const GLubyte squareColors[] = {
        255	,255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
    };
    
    
    static const GLfloat texCoords[] = {
        0, 1,
        1, 1,
        0, 0,
        1, 0
    };
    
    glPushMatrix();
    
    // Move touches to the right place
    glTranslatef(ball.location.x, ball.location.y, 0);
    
    glColorPointer(4, GL_UNSIGNED_BYTE, 0, squareColors);
    glEnableClientState(GL_COLOR_ARRAY);
    glVertexPointer(2, GL_FLOAT, 0, squareVertices);
    glEnableClientState(GL_VERTEX_ARRAY);
    
    // texture coordinate
    glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
    
    
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    
    
    glPopMatrix();
    
}


void UI_Projectiles::draw()
{
    list<ExportedProjectile> points =  m_pModel->getProjectiles();
    
    glBindTexture(GL_TEXTURE_2D, g_texture[1]);
    
    // Draw projectiles
    for( list<ExportedProjectile>::iterator iter = points.begin(); iter!= points.end(); ++iter) {
        drawBall(*iter);
    }
 
    // Get new collisions
    list<Collision> collisions = m_pModel->getCollisions();
    for (list<Collision>::iterator iter = collisions.begin(); iter!= collisions.end(); ++iter) {
        Collision col = *iter;
        if (col.type == COL_EXPLOSION) {
            UI_Collision* uiCol = new UI_Collision(m_pModel, col.location, m_textureExp);
            m_uiCollsions.push_back(uiCol);
        }

    }
    
    // Draw collisions
    for (list<UI_Collision*>::iterator iter = m_uiCollsions.begin(); iter!= m_uiCollsions.end(); ++iter) {
        UI_Collision* pCol = *iter;
        
        pCol->draw();
        
        // delete if done
        if(pCol->isDone()) {
            delete pCol; 
            iter = m_uiCollsions.erase(iter);
        }
    }
}

void UI_Collision::draw()
{
    if( m_done ) {
        return;
    }
    
    // Can grow/shrink the size of the explosion.
    GLfloat factor = 0.5*m_scale;
    
    GLfloat squareVertices[] = {
        factor*-1.0f, factor*-1.0f,
        factor*1.0f,  factor*-1.0f,
        factor*-1.0f, factor*1.0f,
        factor*1.0f,  factor*1.0f
    };
    
    // The alpha channel value is dynamic.
    GLubyte squareColors[] = {
        255, 255, 255, m_intensity*255,
        255, 255, 255, m_intensity*255,
        255, 255, 255, m_intensity*255,
        255, 255, 255, m_intensity*255,
    };
    
    
    
    static const GLfloat texCoords[] = {
        0, 1,
        1, 1,
        0, 0,
        1, 0
    };
    
    glPushMatrix();
    
    glBindTexture(GL_TEXTURE_2D, g_texture[2]);
    
    // Move touches to the right place
    glTranslatef(m_location.x, m_location.y, 0);
    
    glColorPointer(4, GL_UNSIGNED_BYTE, 0, squareColors);
    glEnableClientState(GL_COLOR_ARRAY);
    glVertexPointer(2, GL_FLOAT, 0, squareVertices);
    glEnableClientState(GL_VERTEX_ARRAY);
    
    // texture coordinate
    glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
    
    
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
    
    
    glPopMatrix();
    
    // Grow the explosion, and make it more tranparent.
    m_intensity -= 1.0/TTL;
    m_scale *= 1.05;
    
    // Completely transparent ==> the explosion is finished.
    if(m_intensity < 0) {
        m_done = true;
    }
    
}
