/*
 *  ui_slingshot.cpp
 *
 *  Visual representaiton of the sonic slingshot.
 *  It shows touch points and a vibrating string.
 *
 *  Created by Roy Fejgin on 2/1/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "ui_slingshot.h"


void UI_SlingShot::draw()
{
    SlingState state =  m_pModel->getSlingShotState();
    Float32 vibration = 0;
    CGPoint p1,p2,p3;
    switch (state) {
        case SL_OFF:
            return;
            break;
        case SL_IDLE:
            p1 = m_pModel->getSlingPoint(0);
            drawTouch(p1);
            break;
        case SL_STRECHED:
        case SL_VIBRATING:
            p1 = m_pModel->getSlingPoint(0);
            p2 = m_pModel->getSlingPoint(1);
            vibration = m_pModel->getStringAmplitude();

            drawCord(p1,p2,vibration);
            drawTouch(p1);
            drawTouch(p2);
            
            break;
        case SL_SLING:
            p1 = m_pModel->getSlingPoint(0);
            p2 = m_pModel->getSlingPoint(1);
            p3 = m_pModel->getSlingPoint(2);
            drawCord(p1,p3,0);
            drawCord(p3,p2,0);
            drawTouch(p1);
            drawTouch(p2);
            drawTouch(p3, true);
            break;
            
        default:
            
            assert(0); // not impelmented
            break;
    }
    
}


void UI_SlingShot::drawCord(CGPoint p1, CGPoint p2, Float32 vibration)
{
    static float t = 0;
    
    // Disable texture mapping
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
    

    Vector3D v1,v2, vcord, vnormal;
    v1.x = p1.x;
    v1.y = p1.y;
    v2.x = p2.x;
    v2.y = p2.y;
    
    vcord = v1-v2;
    
    // Vector that is normal to the cord
    vnormal.x = -vcord.y;
    vnormal.y = vcord.x;
    vnormal.z = 0;
    
    // Vibrate the cord
    vnormal.normalize();
    vnormal*= 0.04*sin(2*t)*vibration;
    t+=1;
    GLfloat line[] = {
        p1.x+vnormal.x, p1.y+vnormal.y,
        p2.x+vnormal.x, p2.y+vnormal.y,
    };

    // Yellow
    GLubyte col[] = {
        255,245,0,255,
        255,245,0,255
    };
    
    glPushMatrix();
    
    glColorPointer(4, GL_UNSIGNED_BYTE, 0, col);
    glEnableClientState(GL_COLOR_ARRAY);

    glVertexPointer(2, GL_FLOAT, 0, line);
    glEnableClientState(GL_VERTEX_ARRAY);
        
    glLineWidth(5.);
    glDrawArrays(GL_LINES, 0, 2);
    
    glPopMatrix();
    
    // Re-enable texture mapping
    glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
}



void UI_SlingShot::drawTouch(CGPoint point, bool isProj)
{
    glBindTexture(GL_TEXTURE_2D, g_texture[0]);

    GLfloat factor;
    if (!isProj) {
        factor = 0.8;
    }
    else {
        factor = 0.4;
    }

    GLfloat squareVertices[] = {
        factor*-0.5f, factor*-.5f,
        factor*0.5f,  factor*-.5f,
        factor*-0.5f, factor*.5f,
        factor*0.5f,  factor*.5f
    };

    static const GLubyte squareColors[] = {
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
    };
    
    
    static const GLfloat texCoords[] = {
        0, 1,
        1, 1,
        0, 0,
        1, 0
    };
    
    glPushMatrix();

    // Move touches to the right place
    glTranslatef(point.x, point.y, 0);
    
    glColorPointer(4, GL_UNSIGNED_BYTE, 0, squareColors);
    glEnableClientState(GL_COLOR_ARRAY);
    glVertexPointer(2, GL_FLOAT, 0, squareVertices);
    glEnableClientState(GL_VERTEX_ARRAY);
    
    // texture coordinate
    glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
    
    
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    
    glPopMatrix();
    
}



